@@ -38,7 +38,7 @@
 #define UGREP_HPP
 
 // ugrep version
-#define UGREP_VERSION "4.0.5"
+#define UGREP_VERSION "4.1.0"
 
 // disable mmap because mmap is almost always slower than the file reading speed improvements since 3.0.0
 #define WITH_NO_MMAP
@@ -339,7 +339,7 @@ inline const char *utf8skipn(const char *s, size_t n, size_t k)
 # ifdef OS_WIN
 #  define DEFAULT_PAGER_COMMAND "more"
 # else
-#  define DEFAULT_PAGER_COMMAND "less -R"
+#  define DEFAULT_PAGER_COMMAND "less"
 # endif
 #endif
 
